//{{NO_DEPENDENCIES}}
// Microsoft Visual C++ generated include file.
// Used by pc-das.rc
//
#define IDR_MANIFEST                    1
#define IDD_PCDAS_DIALOG                102
#define IDR_MAINFRAME                   128
#define IDC_kftPortSet                  1000
#define IDC_parosciPortSet              1001
#define IDC_wfPortSet                   1002
#define IDC_IPADDRESS1                  1003
#define IDC_resonIPAddrs                1003
#define IDC_kftPort                     1004
#define IDC_parosciPort                 1005
#define IDC_winfrogPort                 1006
#define IDC_kftPortSet2                 1007
#define IDC_resonAddrSet                1007
#define IDC_rockAddrSet                 1008
#define IDC_gpsList                     1009
#define IDC_wfList                      1010
#define IDC_parosciList                 1011
#define IDC_kft_mode                    1012
#define IDC_CHECK1                      1013
#define IDC_lat_lon_valid               1013
#define IDC_kft_gpsProcessFlag          1014
#define IDC_kft_gpsRejectFlag           1015
#define IDC_kft_dvlProcessFlag          1016
#define IDC_kft_dvlRejectFlag           1017
#define IDC_kft_zuptProcessFlag         1018
#define IDC_kft_alignNoPosFlag          1019
#define IDC_kft_validDVLHeightFlag      1020
#define IDC_kft_Counter                 1021
#define IDC_kft_UTCSecs                 1022
#define IDC_kft_TimeSource              1023
#define IDC_kft_xrate                   1024
#define IDC_kft_yrate                   1025
#define IDC_kft_zrate                   1026
#define IDC_kft_depthLoopOpen           1027
#define IDC_kft_lat                     1028
#define IDC_kft_lon                     1029
#define IDC_kft_depth                   1030
#define IDC_kft_altitude                1031
#define IDC_CHECK2                      1032
#define IDC_kft_nxvel                   1033
#define IDC_kft_nyvel                   1034
#define IDC_kft_nzvel                   1035
#define IDC_CHECK3                      1036
#define IDC_CHECK4                      1037
#define IDC_CHECK5                      1038
#define IDC_CHECK6                      1039
#define IDC_alignRestart                1039
#define IDC_kft_roll                    1040
#define IDC_kft_pitch                   1041
#define IDC_kft_heading                 1042
#define IDC_kft_xacc                    1043
#define IDC_kft_yacc                    1044
#define IDC_kft_zacc                    1045
#define IDC_CHECK7                      1046
#define IDC_CHECK8                      1047
#define IDC_CHECK9                      1048
#define IDC_gpsAidEnable                1048
#define IDC_CHECK10                     1049
#define IDC_dopAidEnab                  1049
#define IDC_dopAidEnable                1049
#define IDC_CHECK11                     1050
#define IDC_CHECK12                     1051
#define IDC_dopPingEnable               1051
#define IDC_EDIT1                       1052
#define IDC_EDIT2                       1053
#define IDC_EDIT3                       1054
#define IDC_EDIT4                       1055
#define IDC_EDIT5                       1056
#define IDC_EDIT6                       1057
#define IDC_kft_monitor                 1058
#define IDC_CHECK13                     1059
#define IDC_CHECK14                     1060
#define IDC_CHECK15                     1061
#define IDC_gpsDepthDamp                1061
#define IDC_CHECK16                     1062
#define IDC_CHECK17                     1063
#define IDC_rockIPAddrs                 1064
#define IDC_LIST1                       1065

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        129
#define _APS_NEXT_COMMAND_VALUE         32771
#define _APS_NEXT_CONTROL_VALUE         1066
#define _APS_NEXT_SYMED_VALUE           101
#endif
#endif
