//============================================================================
/// Summary  : Reson 6046 Driver
/// Filename : Reson6046Driver.h
/// Author   : Reed Christeson, reed@bluefinrobotics.com
/// Project  : Reson 6046 Driver
/// Revision : 0.0
/// Created  : 2002.03.21
/// Modified : 2002.03.21
//============================================================================

#ifndef _RESON7K_H
#define _RESON7K_H

// System Includes.
#include <string.h>
#include <math.h>   // for ceil function

#include "SimpleSocket.h"
#include "Reson7KDataTypes.h"
#include "Reson7KCommands.h"
#include "Reson7kCommand.h"

/// Class Reson6046Driver
class Reson7K
{
 public:
    Reson7K(char *address);
    ~Reson7K();

    
    // Multibeam Remote Command
    int MB_RemoteCommand(int cmdID,
			 float cmdParam = 0.,
			 float cmdParam1 = 0.,
			 float cmdParam2 = 0.,
			 float cmdParam3 = 0.,
       int   iParam = 0);
	int Reson7K::MB_RemoteCommand(int cmdIn, char *recordData);
     // Socket Functions.
    SimpleSocket *_mbDataSocket;

 protected:

 private:
    // Framework functions.



};

#endif
