//=========================================================================
// Summary  : Reson 6046
// Filename : Reson6046Command.h
// Author   : Reed Christenson reed@bluefinrobotics.com
// Project  : Reson 6046 Driver
// Revision : 0.0
// Created  : 2002.03.21
// Modified : 2002.03.21
//=========================================================================
// Description : Reson 6046
//=========================================================================

#ifndef _DATABROADCASTERCOMMAND_H
#define _DATABROADCASTERCOMMAND_H

class Reson6046Command {
 public:
    enum Requests {
	None = 3,
	DataLoggingOn,
	DataLoggingOff,
	// PLC Command ID 0
	StartSubsystem,
	StopSubsystem,
	SetPulseFilename,
	SetPingRange,
	SetPingRate,
	SetPingPower,
	// PLC Command ID 1
	StartLogging,
	StopLogging,
	// PLC Command ID 4
	GetVersion,
	// PLC Command ID 10
	GetSonarStatus,
	// PLC Command 11
	SetVerboseMode,
	// PLC Command ID 14
	GetModules,
	// PLC Command ID 15
	GetRunList,
	// PLC Command ID 20
	SetTime,

	// MB Remote command 1003
	SetMBRange,
	// MB Remote command 1004
	SetMBPingRate,
	// MB Remote command 1005
	SetMBTxPower,
	// MB Remote command 1008
	SetMBRxGain,
	// MB Remote command 1006
	SetMBPulseWidth,
	// MB Remote command 1010

  // MB Remote Command 7610
	SetMBSoundVelocity,
  // MB Remote Command 7611
	SetMBAbsorptionLoss,
  // MB Remote Command 7612
	SetMBSpreadingLoss,

	SetMBBottomDetectionFilters,
	UseDepthRangeMBBottomDetect,
	
	SetMBSnippetControl,
	// MB Remote command 1100
	MBStartPing,
	// MB Remote command 1101
	MBStopPing,
	
	SetSBPDuration,
	SetSBPGain,
	
	RecalcInterval,
	RecalcParams,
	Start7KCenterLogging,
	Stop7KCenterLogging,
	Position=1003,
	Altitude=1006,
	Depth=1008,
	RollPitchHeave=1012,
	Heading=1013
    };

 protected:

 private:

};

#endif
