// Microsoft Visual C++ generated resource script.
//
#include "resource.h"

#define APSTUDIO_READONLY_SYMBOLS
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 2 resource.
//
#include "afxres.h"

/////////////////////////////////////////////////////////////////////////////
#undef APSTUDIO_READONLY_SYMBOLS

/////////////////////////////////////////////////////////////////////////////
// English (U.S.) resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENU)
#ifdef _WIN32
LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_US
#pragma code_page(1252)
#endif //_WIN32

#ifdef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// TEXTINCLUDE
//

1 TEXTINCLUDE 
BEGIN
    "resource.h\0"
END

2 TEXTINCLUDE 
BEGIN
    "#include ""afxres.h""\r\n"
    "\0"
END

3 TEXTINCLUDE 
BEGIN
    "#define _AFX_NO_SPLITTER_RESOURCES\r\n"
    "#define _AFX_NO_OLE_RESOURCES\r\n"
    "#define _AFX_NO_TRACKER_RESOURCES\r\n"
    "#define _AFX_NO_PROPERTY_RESOURCES\r\n"
    "\r\n"
    "#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENU)\r\n"
    "LANGUAGE 9, 1\r\n"
    "#pragma code_page(1252)\r\n"
    "#include ""res\\pc-das.rc2""  // non-Microsoft Visual C++ edited resources\r\n"
    "#include ""afxres.rc""         // Standard components\r\n"
    "#endif\r\n"
    "\0"
END

#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// Icon
//

// Icon with lowest ID value placed first to ensure application icon
// remains consistent on all systems.
IDR_MAINFRAME           ICON                    "res\\pc-das.ico"

/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

IDD_PCDAS_DIALOG DIALOGEX 0, 0, 705, 478
STYLE DS_SETFONT | DS_MODALFRAME | DS_FIXEDSYS | WS_POPUP | WS_VISIBLE | 
    WS_CAPTION
EXSTYLE WS_EX_APPWINDOW
CAPTION "pc-das"
FONT 8, "MS Shell Dlg", 0, 0, 0x1
BEGIN
    DEFPUSHBUTTON   "OK",IDOK,574,458,50,14
    PUSHBUTTON      "Cancel",IDCANCEL,626,458,50,14
    CONTROL         "Kearfott Port",IDC_kftPortSet,"Button",BS_OWNERDRAW | 
                    WS_TABSTOP,9,418,46,14
    CONTROL         "Parosci Port",IDC_parosciPortSet,"Button",BS_OWNERDRAW | 
                    WS_TABSTOP,8,437,46,14
    CONTROL         "Winfrog Port",IDC_wfPortSet,"Button",BS_OWNERDRAW | 
                    WS_TABSTOP,7,455,46,14
    COMBOBOX        IDC_kftPort,61,420,27,42,CBS_DROPDOWN | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_parosciPort,60,437,27,42,CBS_DROPDOWN | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    COMBOBOX        IDC_winfrogPort,59,455,27,42,CBS_DROPDOWN | CBS_SORT | 
                    WS_VSCROLL | WS_TABSTOP
    GROUPBOX        "Navigation Data",IDC_STATIC,0,15,338,259
    GROUPBOX        "DVL Data",IDC_STATIC,342,15,334,259
    GROUPBOX        "I/O Config",IDC_STATIC,0,409,335,64
    EDITTEXT        IDC_kft_mode,67,28,112,12,ES_AUTOHSCROLL
    LTEXT           "Mode",IDC_STATIC,7,28,33,13
    LTEXT           "Open > 1 min",IDC_STATIC,7,55,50,13
    LTEXT           "GPS Process",IDC_STATIC,7,68,50,13
    LTEXT           "GPS Reject",IDC_STATIC,7,82,50,13
    LTEXT           "DVL Process",IDC_STATIC,7,95,50,13
    LTEXT           "DVL Reject",IDC_STATIC,7,109,50,13
    LTEXT           "ZUPT Process",IDC_STATIC,7,122,50,13
    LTEXT           "Align w/o Pos",IDC_STATIC,7,134,50,13
    LTEXT           "Valid DVL",IDC_STATIC,7,148,50,13
    LTEXT           "50 Hz Cycle",IDC_STATIC,7,172,52,13
    LTEXT           "UTC Sys (sec)",IDC_STATIC,7,186,47,13
    LTEXT           "Time Source",IDC_STATIC,7,199,52,13
    LTEXT           "X Ang Rate (d/s)",IDC_STATIC,9,233,96,13
    LTEXT           "Y Ang Rate (d/s)",IDC_STATIC,9,246,96,13
    LTEXT           "Z Ang Rate (d/s)",IDC_STATIC,9,260,96,13
    EDITTEXT        IDC_kft_depthLoopOpen,67,55,31,12,ES_AUTOHSCROLL
    EDITTEXT        IDC_kft_gpsProcessFlag,67,68,31,12,ES_AUTOHSCROLL
    EDITTEXT        IDC_kft_gpsRejectFlag,67,81,31,12,ES_AUTOHSCROLL
    EDITTEXT        IDC_kft_dvlProcessFlag,67,95,31,12,ES_AUTOHSCROLL
    EDITTEXT        IDC_kft_dvlRejectFlag,67,108,31,12,ES_AUTOHSCROLL
    EDITTEXT        IDC_kft_zuptProcessFlag,67,121,31,12,ES_AUTOHSCROLL
    EDITTEXT        IDC_kft_alignNoPosFlag,67,134,31,12,ES_AUTOHSCROLL
    EDITTEXT        IDC_kft_validDVLHeightFlag,67,148,31,12,ES_AUTOHSCROLL
    EDITTEXT        IDC_kft_Counter,67,172,73,12,ES_AUTOHSCROLL | ES_NUMBER
    EDITTEXT        IDC_kft_UTCSecs,67,186,73,12,ES_AUTOHSCROLL | ES_NUMBER
    EDITTEXT        IDC_kft_TimeSource,67,199,73,12,ES_AUTOHSCROLL
    EDITTEXT        IDC_kft_xrate,67,231,73,12,ES_AUTOHSCROLL | ES_NUMBER
    EDITTEXT        IDC_kft_yrate,67,245,73,12,ES_AUTOHSCROLL | ES_NUMBER
    EDITTEXT        IDC_kft_zrate,67,258,73,12,ES_AUTOHSCROLL | ES_NUMBER
    EDITTEXT        IDC_kft_lat,219,55,95,12,ES_AUTOHSCROLL | ES_NUMBER
    EDITTEXT        IDC_kft_lon,219,68,95,12,ES_AUTOHSCROLL | ES_NUMBER
    EDITTEXT        IDC_kft_depth,219,81,95,12,ES_AUTOHSCROLL | ES_NUMBER
    EDITTEXT        IDC_kft_altitude,219,94,95,12,ES_AUTOHSCROLL | ES_NUMBER
    LTEXT           "Lat (deg)",IDC_STATIC,186,55,30,13
    LTEXT           "Lon (deg)",IDC_STATIC,186,68,32,13
    LTEXT           "Depth (m)",IDC_STATIC,184,81,33,13
    LTEXT           "Alt (m)",IDC_STATIC,185,94,33,13
    EDITTEXT        IDC_kft_nxvel,219,124,95,12,ES_AUTOHSCROLL | ES_NUMBER
    EDITTEXT        IDC_kft_nyvel,219,137,95,12,ES_AUTOHSCROLL | ES_NUMBER
    EDITTEXT        IDC_kft_nzvel,219,150,95,12,ES_AUTOHSCROLL | ES_NUMBER
    LTEXT           "Nx vel (cm/s)",IDC_STATIC,174,124,44,13
    LTEXT           "Ny vel (cm/s)",IDC_STATIC,174,135,44,13
    LTEXT           "Nz vel (cm/s)",IDC_STATIC,174,150,44,13
    EDITTEXT        IDC_kft_roll,219,172,95,12,ES_AUTOHSCROLL | ES_NUMBER
    EDITTEXT        IDC_kft_pitch,219,185,95,12,ES_AUTOHSCROLL | ES_NUMBER
    EDITTEXT        IDC_kft_heading,219,199,95,12,ES_AUTOHSCROLL | ES_NUMBER
    LTEXT           "Roll (deg)",IDC_STATIC,175,172,44,13
    LTEXT           "Pitch(deg)",IDC_STATIC,175,185,44,13
    LTEXT           "Head(deg)",IDC_STATIC,175,199,44,13
    EDITTEXT        IDC_kft_xacc,218,230,95,12,ES_AUTOHSCROLL | ES_NUMBER
    EDITTEXT        IDC_kft_yacc,218,242,95,12,ES_AUTOHSCROLL | ES_NUMBER
    EDITTEXT        IDC_kft_zacc,218,257,95,12,ES_AUTOHSCROLL | ES_NUMBER
    LTEXT           "X Acc (m/s2)",IDC_STATIC,173,230,44,13
    LTEXT           "Y Acc (m/S2)",IDC_STATIC,173,244,44,13
    LTEXT           "Z Acc (m/S2)",IDC_STATIC,173,257,44,13
    CONTROL         "",IDC_resonIPAddrs,"SysIPAddress32",WS_TABSTOP,146,421,
                    133,13
    CONTROL         "Reson IP",IDC_resonAddrSet,"Button",BS_OWNERDRAW | 
                    WS_TABSTOP,95,420,46,14
    CONTROL         "",IDC_gpsList,"SysListView32",LVS_REPORT | 
                    LVS_ALIGNLEFT | WS_BORDER | WS_TABSTOP,7,275,319,40
    CONTROL         "",IDC_wfList,"SysListView32",LVS_REPORT | LVS_ALIGNLEFT | 
                    WS_BORDER | WS_TABSTOP,7,319,319,40
    GROUPBOX        "Operational Input Message",IDC_STATIC,343,277,333,178
    CONTROL         "Lat/Lon Valid",IDC_lat_lon_valid,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,352,292,60,8
    CONTROL         "Pres Depth Valid",IDC_CHECK2,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,352,301,70,8
    CONTROL         "Heading Valid",IDC_CHECK3,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,352,310,70,8
    CONTROL         "Salinity Valid",IDC_CHECK4,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,352,319,70,8
    CONTROL         "Stationary Align",IDC_CHECK5,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,352,328,70,8
    CONTROL         "Align Restart",IDC_alignRestart,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,352,337,70,8
    CONTROL         "ZUPT Enable",IDC_CHECK7,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,352,346,70,8
    CONTROL         "Commanded ZUPT",IDC_CHECK8,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,352,355,70,8
    CONTROL         "GPS Aid Enable",IDC_gpsAidEnable,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,352,364,70,8
    CONTROL         "Dop Aid Enable",IDC_dopAidEnable,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,352,373,70,8
    CONTROL         "Shutdown Command",IDC_CHECK11,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,352,382,78,8
    CONTROL         "Dop Ping Enable",IDC_dopPingEnable,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,352,391,78,8
    EDITTEXT        IDC_EDIT1,559,297,87,11,ES_AUTOHSCROLL
    LTEXT           "Latitude",IDC_STATIC,520,298,29,11
    EDITTEXT        IDC_EDIT2,559,310,87,11,ES_AUTOHSCROLL
    LTEXT           "Longitude",IDC_STATIC,520,310,38,11
    EDITTEXT        IDC_EDIT3,559,323,87,11,ES_AUTOHSCROLL
    LTEXT           "Pres Depth (m)",IDC_STATIC,506,323,51,11
    EDITTEXT        IDC_EDIT4,559,335,87,11,ES_AUTOHSCROLL
    LTEXT           "Salinity (ppt)",IDC_STATIC,505,335,53,11
    EDITTEXT        IDC_EDIT5,559,347,87,11,ES_AUTOHSCROLL
    LTEXT           "Pos Qual (m)",IDC_STATIC,505,347,53,11
    EDITTEXT        IDC_EDIT6,559,360,87,11,ES_AUTOHSCROLL
    LTEXT           "DVL Track Mode",IDC_STATIC,504,360,53,11
    CONTROL         "Force Pos Fix",IDC_CHECK13,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,352,400,60,8
    CONTROL         "Align w/o Position",IDC_CHECK14,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,352,409,76,8
    CONTROL         "GPS Depth Damp",IDC_gpsDepthDamp,"Button",
                    BS_AUTOCHECKBOX | WS_TABSTOP,352,418,76,8
    CONTROL         "Land Align",IDC_CHECK16,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,352,426,76,8
    CONTROL         "Dynamic Depth",IDC_CHECK17,"Button",BS_AUTOCHECKBOX | 
                    WS_TABSTOP,352,434,76,8
    LTEXT           "Monitor",IDC_STATIC,7,43,33,13
    EDITTEXT        IDC_kft_monitor,67,42,31,12,ES_AUTOHSCROLL
    CONTROL         "",IDC_parosciList,"SysListView32",LVS_REPORT | 
                    LVS_ALIGNLEFT | WS_BORDER | WS_TABSTOP,7,363,319,38
    CONTROL         "Rock IP",IDC_rockAddrSet,"Button",BS_OWNERDRAW | 
                    WS_TABSTOP,95,438,46,14
    CONTROL         "",IDC_rockIPAddrs,"SysIPAddress32",WS_TABSTOP,145,439,
                    133,13
END


/////////////////////////////////////////////////////////////////////////////
//
// Version
//

VS_VERSION_INFO VERSIONINFO
 FILEVERSION 1,0,0,1
 PRODUCTVERSION 1,0,0,1
 FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x4L
 FILETYPE 0x1L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904e4"
        BEGIN
            VALUE "CompanyName", "TODO: <Company name>"
            VALUE "FileDescription", "TODO: <File description>"
            VALUE "FileVersion", "1.0.0.1"
            VALUE "InternalName", "pc-das.exe"
            VALUE "LegalCopyright", "TODO: (c) <Company name>.  All rights reserved."
            VALUE "OriginalFilename", "pc-das.exe"
            VALUE "ProductName", "TODO: <Product name>"
            VALUE "ProductVersion", "1.0.0.1"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1252
    END
END


/////////////////////////////////////////////////////////////////////////////
//
// DESIGNINFO
//

#ifdef APSTUDIO_INVOKED
GUIDELINES DESIGNINFO 
BEGIN
    IDD_PCDAS_DIALOG, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 676
        TOPMARGIN, 7
        BOTTOMMARGIN, 471
    END
END
#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// RT_MANIFEST
//

IDR_MANIFEST            RT_MANIFEST             "res\\pc-das.manifest"
#endif    // English (U.S.) resources
/////////////////////////////////////////////////////////////////////////////



#ifndef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 3 resource.
//
#define _AFX_NO_SPLITTER_RESOURCES
#define _AFX_NO_OLE_RESOURCES
#define _AFX_NO_TRACKER_RESOURCES
#define _AFX_NO_PROPERTY_RESOURCES

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENU)
LANGUAGE 9, 1
#pragma code_page(1252)
#include "res\pc-das.rc2"  // non-Microsoft Visual C++ edited resources
#include "afxres.rc"         // Standard components
#endif

/////////////////////////////////////////////////////////////////////////////
#endif    // not APSTUDIO_INVOKED

